attribute vec4 tangent;

uniform vec2 uOffset;
uniform vec2 uRepeat;

uniform bool enableDisplacement;

#ifdef VERTEX_TEXTURES
	uniform sampler2D tDisplacement;
	uniform float uDisplacementScale;
	uniform float uDisplacementBias;
#endif

varying vec3 vTangent;
varying vec3 vBinormal;
varying vec3 vNormal;
varying vec2 vUv;

varying vec3 vWorldPosition;
varying vec3 vViewPosition;

#ifdef USE_LOGDEPTHBUF
	#ifdef USE_LOGDEPTHBUF_EXT
		varying float vFragDepth;
	#endif
	uniform float logDepthBufFC;
#endif

void main() {

	// vNormal = normalize( normalMatrix * normal );
	// vTangent = normalize( normalMatrix * tangent.xyz );
	vNormal = normalize( normal );
	vTangent = normalize( tangent.xyz );
	vBinormal = normalize( cross( vNormal, vTangent ) * tangent.w );

	vUv = uv * uRepeat + uOffset;

	vec3 displacedPosition;

	#ifdef VERTEX_TEXTURES
		if ( enableDisplacement ) {
			vec3 dv = texture2D( tDisplacement, uv ).xyz;
			float df = uDisplacementScale * dv.x + uDisplacementBias;
			displacedPosition = position + normalize( normal ) * df;
		} else {
			displacedPosition = position;
		}
	#else
		displacedPosition = position;
	#endif
	vec4 mvPosition = modelViewMatrix * vec4( displacedPosition, 1.0 );
	vec4 worldPosition = modelMatrix * vec4( displacedPosition, 1.0 );

	gl_Position = projectionMatrix * mvPosition;

	//===========================================
	// THREE.ShaderChunk[ "logdepthbuf_vertex" ],
	//===========================================
	#ifdef USE_LOGDEPTHBUF
		gl_Position.z = log2(max(1e-6, gl_Position.w + 1.0)) * logDepthBufFC;
	
		#ifdef USE_LOGDEPTHBUF_EXT
			vFragDepth = 1.0 + gl_Position.w;
		#else
			gl_Position.z = (gl_Position.z - 1.0) * gl_Position.w;
		#endif
	#endif
	//===========================================
	//===========================================
	//===========================================

	vWorldPosition = worldPosition.xyz;
	vViewPosition = -mvPosition.xyz;

}
